package it.unibo.cs.helm.uwobo;

import java.io.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;
import javax.xml.transform.*;
import org.xml.sax.*;

/*
*
* usage:
*   http://aristotele/helm/servlet/uwobo/help
*   http://aristotele/helm/servlet/uwobo/add?xsluri=&key=
*   http://aristotele/helm/servlet/uwobo/remove[?key=]
*   http://aristotele/helm/servlet/uwobo/list
*   http://aristotele/helm/servlet/uwobo/reload[?key=]
*   http://aristotele/helm/servlet/uwobo/update[?key=]
*   http://aristotele/helm/servlet/uwobo/apply?xmluri=[&key=]+[&par.=]*
*
* example:
*   http://aristotele/helm/servlet/uwobo/add?xsluri=file:///D:/Archivio/Progetti/helm/resources/xsl/foo1.xsl&key=foo1
*   http://aristotele/helm/servlet/uwobo/add?xsluri=file:///D:/Archivio/Progetti/helm/resources/xsl/foo2.xsl&key=foo2
*   http://aristotele/helm/servlet/uwobo/apply?xmluri=file:///D:/Archivio/Progetti/helm/resources/xsl/foo.xml&key=foo1&key=foo2
*
* installation notes (Tomcat):
*	replace parser.jar and jaxp.jar from /lib with xerces.jar
*
*	add in conf/server.xml
*		<Context path="/helm" 
*			docBase="webapps/helm" 
*			crossContext="false"
*			debug="0" 
*			reloadable="true" > 
*		</Context>
*
*	add in uriworkermap.properties
*		/helm/*=ajp12
*
* bugs:
*	directory base stylesheet inclusi
*
* @author Riccardo Solmi
*/
public class Servlet extends HttpServlet {
	public static final String[] usage = {
		"http://<i>hostname</i>/helm/servlet/uwobo/help",
		"http://<i>hostname</i>/helm/servlet/uwobo/add?xsluri=<i>stylesheet</i>&key=<i>name</i>",
		"http://<i>hostname</i>/helm/servlet/uwobo/remove[?key=<i>name</i>]",
		"http://<i>hostname</i>/helm/servlet/uwobo/list",
		"http://<i>hostname</i>/helm/servlet/uwobo/reload[?key=<i>name</i>]",
		"http://<i>hostname</i>/helm/servlet/uwobo/update[?key=<i>name</i>]",
		"http://<i>hostname</i>/helm/servlet/uwobo/apply?xmluri=<i>xmldata</i>[&key=<i>name</i>]+[&par.<i>name</i>=<i>value</i>]*"
	};
	public static final String help;
	
	static {
		StringBuffer sb = new StringBuffer();
		sb.append("<ul>");
		for (int i=0; i<usage.length; i++)
			sb.append("<li>").append(usage[i]).append("</li>");
		sb.append("</ul>");
		help = sb.toString();
	}

	private Server server;

	public void init(ServletConfig config) throws ServletException {
		super.init(config);

		System.out.println("UWOBO init");
		server = new Server();
	}

	public void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
System.out.println("UWOBO "+request.getPathInfo());
		ServletOutputStream out;

		try {
			final String cmd = request.getPathInfo();
			if (cmd == null) {
				sendError(response, HttpServletResponse.SC_NOT_FOUND, "unknown command", help);
				return;
			}
			if (cmd.equals("/add")) {
				final String filename = request.getParameter("xsluri");
				if (filename == null) {
					sendError(response, HttpServletResponse.SC_BAD_REQUEST, "bad parameters", usage[1]);
					return;
				}
				final String key = request.getParameter("key");
				server.add(filename, key);
			} else if (cmd.equals("/apply")) {
				final String infile = request.getParameter("xmluri");
				final String[] keys = request.getParameterValues("key");
				if (infile==null || keys==null) {
					sendError(response, HttpServletResponse.SC_BAD_REQUEST, "bad parameters", usage[6]);
					return;
				}

				final List params = new ArrayList();
				final Enumeration e = request.getParameterNames();
				while (e.hasMoreElements()) {
					String param = (String)e.nextElement();
					if (param.startsWith("param."))
						params.add(new String[]{param.substring(6), request.getParameter(param)});
				}
					
				response.setContentType(server.getContentType(keys[keys.length-1]));
				out = response.getOutputStream();
				server.apply(infile, out, keys, params);
				out.close();
				return;
			} else if (cmd.equals("/remove")) {
				final String key = request.getParameter("key");
				if (key == null)
					server.removeAll();
				else
					server.remove(key);
			} else if (cmd.equals("/list")) {
				Iterator i = server.list().iterator();

				response.setContentType("text/html");
				out = response.getOutputStream();
				out.println("<html><body><h1>Uwobo servlet</h1><p>stylesheet list:</p><ul>");
				while (i.hasNext())
					out.println("<li>"+i.next()+"</li>");
				out.println("</ul></body></html>");
				out.close();
				return;
			} else if (cmd.equals("/reload")) {
				final String key = request.getParameter("key");
				if (key == null)
					server.reloadAll();
				else
					server.reload(key);
			} else if (cmd.equals("/update")) {
				final String key = request.getParameter("key");
				if (key == null)
					server.updateAll();
				else
					server.update(key);
			} else if (cmd.equals("/help")) {
				response.setContentType("text/html");
				out = response.getOutputStream();
				out.println("<html><body><h1>"+server.PACKAGE+" servlet - version "+server.VERSION+"</h1>");
				out.println("<b>compiled "+server.DATE+" at "+server.TIME.substring(0,2)+":"+server.TIME.substring(2)+"</b>");
				out.println("<p>usage:</p>"+help+"</body></html>");
				out.close();
				return;
			} else {
				sendError(response, HttpServletResponse.SC_NOT_FOUND, "unknown command", help);
				return;
			}	
		} catch (TransformerConfigurationException tce) {
			sendError(response, HttpServletResponse.SC_BAD_REQUEST, "stylesheet error", tce);
			return;
		} catch (SAXException se) {
			sendError(response, HttpServletResponse.SC_BAD_REQUEST, "stylesheet error", se);
			return;
		} catch (Exception e) {
			sendError(response, HttpServletResponse.SC_BAD_REQUEST, "stylesheet not found", e);
			return;
		}

		response.setContentType("text/html");
		out = response.getOutputStream();
		out.println("<html><body><h1>Uwobo servlet</h1><p>done</p></body></html>");
		out.close();
	}

	private void sendError(HttpServletResponse response, int code, String msg, Exception e) throws IOException {
		String err;
		if (e != null)
			err = "<p>"+e.getMessage()+"</P>";
		else
			err = "";
		response.setContentType("text/html");
		response.sendError(code, "<html><body><h1>Uwobo servlet</h1><p>"+msg+"</p>"+err+"</body></html>");
	}
	
	private void sendError(HttpServletResponse response, int code, String msg, String usage) throws IOException {
		response.setContentType("text/html");
		response.sendError(code, "<html><body><h1>Uwobo servlet</h1><p>"+msg+"</p>usage: "+usage+"</body></html>");
	}
	
	public String getServletInfo() {
		return "The UWOBO servlet";
	}
}
